# -*- coding: utf-8 -*-
"""
A list of Peru regions as `choices` in a formfield.

This exists in this standalone file so that it's only imported into memory
when explicitly needed.
"""
from __future__ import unicode_literals

REGION_CHOICES = (
    ('AMA', 'Amazonas'),
    ('ANC', 'Ancash'),
    ('APU', 'Apurímac'),
    ('ARE', 'Arequipa'),
    ('AYA', 'Ayacucho'),
    ('CAJ', 'Cajamarca'),
    ('CAL', 'Callao'),
    ('CUS', 'Cusco'),
    ('HUV', 'Huancavelica'),
    ('HUC', 'Huánuco'),
    ('ICA', 'Ica'),
    ('JUN', 'Junín'),
    ('LAL', 'La Libertad'),
    ('LAM', 'Lambayeque'),
    ('LIM', 'Lima'),
    ('LOR', 'Loreto'),
    ('MDD', 'Madre de Dios'),
    ('MOQ', 'Moquegua'),
    ('PAS', 'Pasco'),
    ('PIU', 'Piura'),
    ('PUN', 'Puno'),
    ('SAM', 'San Martín'),
    ('TAC', 'Tacna'),
    ('TUM', 'Tumbes'),
    ('UCA', 'Ucayali'),
)
