from django.utils.translation import ugettext_lazy as _

# ISO codes
PROVINCE_CHOICES = (
    ('VAN', _('Antwerp')),
    ('BRU', _('Brussels')),
    ('VOV', _('East Flanders')),
    ('VBR', _('Flemish Brabant')),
    ('WHT', _('Hainaut')),
    ('WLG', _('Liege')),
    ('VLI', _('Limburg')),
    ('WLX', _('Luxembourg')),
    ('WNA', _('Namur')),
    ('WBR', _('Walloon Brabant')),
    ('VWV', _('West Flanders'))
)
