{% extends "databrowse/base_site.html" %}

{% block title %}{{ field.verbose_name|capfirst }} calendar{% endblock %}

{% block content %}

<div id="breadcrumbs"><a href="{{ root_url }}">Home</a> / <a href="{{ model.url }}">{{ model.verbose_name_plural|capfirst }}</a> / <a href="../">Calendars</a> / By {{ field.verbose_name }}</div>

<h1>{{ model.verbose_name_plural|capfirst }} by {{ field.verbose_name }}</h1>

<ul class="objectlist">
{% for year in date_list %}
<li class="{% cycle 'odd' 'even' %}"><a href="{{ year|date:"Y" }}/">{{ year|date:"Y" }}</a></li>
{% endfor %}
</ul>

{% endblock %}
