# -*- coding: utf-8 -*-
"""
An alphabetical list of Norwegian municipalities (fylker) fro use as `choices`
in a formfield.

This exists in this standalone file so that it's on ly imported into memory
when explicitly needed.
"""
from __future__ import unicode_literals

MUNICIPALITY_CHOICES = (
    ('akershus', 'Akershus'),
    ('austagder', 'Aust-Agder'),
    ('buskerud', 'Buskerud'),
    ('finnmark', 'Finnmark'),
    ('hedmark', 'Hedmark'),
    ('hordaland', 'Hordaland'),
    ('janmayen', 'Jan Mayen'),
    ('moreogromsdal', 'Møre og Romsdal'),
    ('nordtrondelag', 'Nord-Trøndelag'),
    ('nordland', 'Nordland'),
    ('oppland', 'Oppland'),
    ('oslo', 'Oslo'),
    ('rogaland', 'Rogaland'),
    ('sognogfjordane', 'Sogn og Fjordane'),
    ('svalbard', 'Svalbard'),
    ('sortrondelag', 'Sør-Trøndelag'),
    ('telemark', 'Telemark'),
    ('troms', 'Troms'),
    ('vestagder', 'Vest-Agder'),
    ('vestfold', 'Vestfold'),
    ('ostfold', 'Østfold')
)
