# -*- coding: utf-8 -*-

"""
An alphabetical list of provinces for use as `choices` in a formfield.

Reference:
http://en.wikipedia.org/wiki/ISO_3166-2:CN
http://en.wikipedia.org/wiki/Province_%28China%29
http://en.wikipedia.org/wiki/Direct-controlled_municipality
http://en.wikipedia.org/wiki/Autonomous_regions_of_China
"""
from __future__ import unicode_literals

CN_PROVINCE_CHOICES = (
    ("anhui", "安徽"),
    ("beijing", "北京"),
    ("chongqing", "重庆"),
    ("fujian", "福建"),
    ("gansu", "甘肃"),
    ("guangdong", "广东"),
    ("guangxi", "广西壮族自治区"),
    ("guizhou", "贵州"),
    ("hainan", "海南"),
    ("hebei", "河北"),
    ("heilongjiang", "黑龙江"),
    ("henan", "河南"),
    ("hongkong", "香港"),
    ("hubei", "湖北"),
    ("hunan", "湖南"),
    ("jiangsu", "江苏"),
    ("jiangxi", "江西"),
    ("jilin", "吉林"),
    ("liaoning", "辽宁"),
    ("macao", "澳门"),
    ("neimongol", "内蒙古自治区"),
    ("ningxia", "宁夏回族自治区"),
    ("qinghai", "青海"),
    ("shaanxi", "陕西"),
    ("shandong", "山东"),
    ("shanghai", "上海"),
    ("shanxi", "山西"),
    ("sichuan", "四川"),
    ("taiwan", "台湾"),
    ("tianjin", "天津"),
    ("xinjiang", "新疆维吾尔自治区"),
    ("xizang", "西藏自治区"),
    ("yunnan", "云南"),
    ("zhejiang", "浙江"),
)
