# -*- coding: utf-8 -*-
"""
A list of Colombian departaments as `choices` in a
formfield.

This exists in this standalone file so that it's only
imported into memory when explicitly needed.
"""
from __future__ import unicode_literals

DEPARTMENT_CHOICES = (
    ('AMA', 'Amazonas'),
    ('ANT', 'Antioquia'),
    ('ARA', 'Arauca'),
    ('ATL', 'Atlántico'),
    ('DC',  'Bogotá'),
    ('BOL', 'Bolívar'),
    ('BOY', 'Boyacá'),
    ('CAL', 'Caldas'),
    ('CAQ', 'Caquetá'),
    ('CAS', 'Casanare'),
    ('CAU', 'Cauca'),
    ('CES', 'Cesar'),
    ('CHO', 'Chocó'),
    ('COR', 'Córdoba'),
    ('CUN', 'Cundinamarca'),
    ('GUA', 'Guainía'),
    ('GUV', 'Guaviare'),
    ('HUI', 'Huila'),
    ('LAG', 'La Guajira'),
    ('MAG', 'Magdalena'),
    ('MET', 'Meta'),
    ('NAR', 'Nariño'),
    ('NSA', 'Norte de Santander'),
    ('PUT', 'Putumayo'),
    ('QUI', 'Quindío'),
    ('RIS', 'Risaralda'),
    ('SAP', 'San Andrés and Providencia'),
    ('SAN', 'Santander'),
    ('SUC', 'Sucre'),
    ('TOL', 'Tolima'),
    ('VAC', 'Valle del Cauca'),
    ('VAU', 'Vaupés'),
    ('VID', 'Vichada'),
)
