# -*- coding: utf-8 -*-
"""
An alphabetical list of Finnish municipalities for use as `choices` in a
formfield.

This exists in this standalone file so that it's only imported into memory
when explicitly needed.
"""

from __future__ import unicode_literals

MUNICIPALITY_CHOICES = (
    ('akaa', "Akaa"),
    ('alajarvi', "Alajärvi"),
    ('alavieska', "Alavieska"),
    ('alavus', "Alavus"),
    ('artjarvi', "Artjärvi"),
    ('asikkala', "Asikkala"),
    ('askola', "Askola"),
    ('aura', "Aura"),
    ('brando', "Brändö"),
    ('eckero', "Eckerö"),
    ('enonkoski', "Enonkoski"),
    ('enontekio', "Enontekiö"),
    ('espoo', "Espoo"),
    ('eura', "Eura"),
    ('eurajoki', "Eurajoki"),
    ('evijarvi', "Evijärvi"),
    ('finstrom', "Finström"),
    ('forssa', "Forssa"),
    ('foglo', "Föglö"),
    ('geta', "Geta"),
    ('haapajarvi', "Haapajärvi"),
    ('haapavesi', "Haapavesi"),
    ('hailuoto', "Hailuoto"),
    ('halsua', "Halsua"),
    ('hamina', "Hamina"),
    ('hammarland', "Hammarland"),
    ('hankasalmi', "Hankasalmi"),
    ('hanko', "Hanko"),
    ('harjavalta', "Harjavalta"),
    ('hartola', "Hartola"),
    ('hattula', "Hattula"),
    ('haukipudas', "Haukipudas"),
    ('hausjarvi', "Hausjärvi"),
    ('heinola', "Heinola"),
    ('heinavesi', "Heinävesi"),
    ('helsinki', "Helsinki"),
    ('hirvensalmi', "Hirvensalmi"),
    ('hollola', "Hollola"),
    ('honkajoki', "Honkajoki"),
    ('huittinen', "Huittinen"),
    ('humppila', "Humppila"),
    ('hyrynsalmi', "Hyrynsalmi"),
    ('hyvinkaa', "Hyvinkää"),
    ('hameenkoski', "Hämeenkoski"),
    ('hameenkyro', "Hämeenkyrö"),
    ('hameenlinna', "Hämeenlinna"),
    ('ii', "Ii"),
    ('iisalmi', "Iisalmi"),
    ('iitti', "Iitti"),
    ('ikaalinen', "Ikaalinen"),
    ('ilmajoki', "Ilmajoki"),
    ('ilomantsi', "Ilomantsi"),
    ('imatra', "Imatra"),
    ('inari', "Inari"),
    ('inkoo', "Inkoo"),
    ('isojoki', "Isojoki"),
    ('isokyro', "Isokyrö"),
    ('jalasjarvi', "Jalasjärvi"),
    ('janakkala', "Janakkala"),
    ('joensuu', "Joensuu"),
    ('jokioinen', "Jokioinen"),
    ('jomala', "Jomala"),
    ('joroinen', "Joroinen"),
    ('joutsa', "Joutsa"),
    ('juankoski', "Juankoski"),
    ('juuka', "Juuka"),
    ('juupajoki', "Juupajoki"),
    ('juva', "Juva"),
    ('jyvaskyla', "Jyväskylä"),
    ('jamijarvi', "Jämijärvi"),
    ('jamsa', "Jämsä"),
    ('jarvenpaa', "Järvenpää"),
    ('kaarina', "Kaarina"),
    ('kaavi', "Kaavi"),
    ('kajaani', "Kajaani"),
    ('kalajoki', "Kalajoki"),
    ('kangasala', "Kangasala"),
    ('kangasniemi', "Kangasniemi"),
    ('kankaanpaa', "Kankaanpää"),
    ('kannonkoski', "Kannonkoski"),
    ('kannus', "Kannus"),
    ('karijoki', "Karijoki"),
    ('karjalohja', "Karjalohja"),
    ('karkkila', "Karkkila"),
    ('karstula', "Karstula"),
    ('karttula', "Karttula"),
    ('karvia', "Karvia"),
    ('kaskinen', "Kaskinen"),
    ('kauhajoki', "Kauhajoki"),
    ('kauhava', "Kauhava"),
    ('kauniainen', "Kauniainen"),
    ('kaustinen', "Kaustinen"),
    ('keitele', "Keitele"),
    ('kemi', "Kemi"),
    ('kemijarvi', "Kemijärvi"),
    ('keminmaa', "Keminmaa"),
    ('kemionsaari', "Kemiönsaari"),
    ('kempele', "Kempele"),
    ('kerava', "Kerava"),
    ('kerimaki', "Kerimäki"),
    ('kesalahti', "Kesälahti"),
    ('keuruu', "Keuruu"),
    ('kihnio', "Kihniö"),
    ('kiikoinen', "Kiikoinen"),
    ('kiiminki', "Kiiminki"),
    ('kinnula', "Kinnula"),
    ('kirkkonummi', "Kirkkonummi"),
    ('kitee', "Kitee"),
    ('kittila', "Kittilä"),
    ('kiuruvesi', "Kiuruvesi"),
    ('kivijarvi', "Kivijärvi"),
    ('kokemaki', "Kokemäki"),
    ('kokkola', "Kokkola"),
    ('kolari', "Kolari"),
    ('konnevesi', "Konnevesi"),
    ('kontiolahti', "Kontiolahti"),
    ('korsnas', "Korsnäs"),
    ('koskitl', "Koski Tl"),
    ('kotka', "Kotka"),
    ('kouvola', "Kouvola"),
    ('kristiinankaupunki', "Kristiinankaupunki"),
    ('kruunupyy', "Kruunupyy"),
    ('kuhmalahti', "Kuhmalahti"),
    ('kuhmo', "Kuhmo"),
    ('kuhmoinen', "Kuhmoinen"),
    ('kumlinge', "Kumlinge"),
    ('kuopio', "Kuopio"),
    ('kuortane', "Kuortane"),
    ('kurikka', "Kurikka"),
    ('kustavi', "Kustavi"),
    ('kuusamo', "Kuusamo"),
    ('kylmakoski', "Kylmäkoski"),
    ('kyyjarvi', "Kyyjärvi"),
    ('karkola', "Kärkölä"),
    ('karsamaki', "Kärsämäki"),
    ('kokar', "Kökar"),
    ('koylio', "Köyliö"),
    ('lahti', "Lahti"),
    ('laihia', "Laihia"),
    ('laitila', "Laitila"),
    ('lapinjarvi', "Lapinjärvi"),
    ('lapinlahti', "Lapinlahti"),
    ('lappajarvi', "Lappajärvi"),
    ('lappeenranta', "Lappeenranta"),
    ('lapua', "Lapua"),
    ('laukaa', "Laukaa"),
    ('lavia', "Lavia"),
    ('lemi', "Lemi"),
    ('lemland', "Lemland"),
    ('lempaala', "Lempäälä"),
    ('leppavirta', "Leppävirta"),
    ('lestijarvi', "Lestijärvi"),
    ('lieksa', "Lieksa"),
    ('lieto', "Lieto"),
    ('liminka', "Liminka"),
    ('liperi', "Liperi"),
    ('lohja', "Lohja"),
    ('loimaa', "Loimaa"),
    ('loppi', "Loppi"),
    ('loviisa', "Loviisa"),
    ('luhanka', "Luhanka"),
    ('lumijoki', "Lumijoki"),
    ('lumparland', "Lumparland"),
    ('luoto', "Luoto"),
    ('luumaki', "Luumäki"),
    ('luvia', "Luvia"),
    ('lansi-turunmaa', "Länsi-Turunmaa"),
    ('maalahti', "Maalahti"),
    ('maaninka', "Maaninka"),
    ('maarianhamina', "Maarianhamina"),
    ('marttila', "Marttila"),
    ('masku', "Masku"),
    ('merijarvi', "Merijärvi"),
    ('merikarvia', "Merikarvia"),
    ('miehikkala', "Miehikkälä"),
    ('mikkeli', "Mikkeli"),
    ('muhos', "Muhos"),
    ('multia', "Multia"),
    ('muonio', "Muonio"),
    ('mustasaari', "Mustasaari"),
    ('muurame', "Muurame"),
    ('mynamaki', "Mynämäki"),
    ('myrskyla', "Myrskylä"),
    ('mantsala', "Mäntsälä"),
    ('mantta-vilppula', "Mänttä-Vilppula"),
    ('mantyharju', "Mäntyharju"),
    ('naantali', "Naantali"),
    ('nakkila', "Nakkila"),
    ('nastola', "Nastola"),
    ('nilsia', "Nilsiä"),
    ('nivala', "Nivala"),
    ('nokia', "Nokia"),
    ('nousiainen', "Nousiainen"),
    ('nummi-pusula', "Nummi-Pusula"),
    ('nurmes', "Nurmes"),
    ('nurmijarvi', "Nurmijärvi"),
    ('narpio', "Närpiö"),
    ('oravainen', "Oravainen"),
    ('orimattila', "Orimattila"),
    ('oripaa', "Oripää"),
    ('orivesi', "Orivesi"),
    ('oulainen', "Oulainen"),
    ('oulu', "Oulu"),
    ('oulunsalo', "Oulunsalo"),
    ('outokumpu', "Outokumpu"),
    ('padasjoki', "Padasjoki"),
    ('paimio', "Paimio"),
    ('paltamo', "Paltamo"),
    ('parikkala', "Parikkala"),
    ('parkano', "Parkano"),
    ('pedersore', "Pedersöre"),
    ('pelkosenniemi', "Pelkosenniemi"),
    ('pello', "Pello"),
    ('perho', "Perho"),
    ('pertunmaa', "Pertunmaa"),
    ('petajavesi', "Petäjävesi"),
    ('pieksamaki', "Pieksämäki"),
    ('pielavesi', "Pielavesi"),
    ('pietarsaari', "Pietarsaari"),
    ('pihtipudas', "Pihtipudas"),
    ('pirkkala', "Pirkkala"),
    ('polvijarvi', "Polvijärvi"),
    ('pomarkku', "Pomarkku"),
    ('pori', "Pori"),
    ('pornainen', "Pornainen"),
    ('porvoo', "Porvoo"),
    ('posio', "Posio"),
    ('pudasjarvi', "Pudasjärvi"),
    ('pukkila', "Pukkila"),
    ('punkaharju', "Punkaharju"),
    ('punkalaidun', "Punkalaidun"),
    ('puolanka', "Puolanka"),
    ('puumala', "Puumala"),
    ('pyhtaa', "Pyhtää"),
    ('pyhajoki', "Pyhäjoki"),
    ('pyhajarvi', "Pyhäjärvi"),
    ('pyhanta', "Pyhäntä"),
    ('pyharanta', "Pyhäranta"),
    ('palkane', "Pälkäne"),
    ('poytya', "Pöytyä"),
    ('raahe', "Raahe"),
    ('raasepori', "Raasepori"),
    ('raisio', "Raisio"),
    ('rantasalmi', "Rantasalmi"),
    ('ranua', "Ranua"),
    ('rauma', "Rauma"),
    ('rautalampi', "Rautalampi"),
    ('rautavaara', "Rautavaara"),
    ('rautjarvi', "Rautjärvi"),
    ('reisjarvi', "Reisjärvi"),
    ('riihimaki', "Riihimäki"),
    ('ristiina', "Ristiina"),
    ('ristijarvi', "Ristijärvi"),
    ('rovaniemi', "Rovaniemi"),
    ('ruokolahti', "Ruokolahti"),
    ('ruovesi', "Ruovesi"),
    ('rusko', "Rusko"),
    ('raakkyla', "Rääkkylä"),
    ('saarijarvi', "Saarijärvi"),
    ('salla', "Salla"),
    ('salo', "Salo"),
    ('saltvik', "Saltvik"),
    ('sastamala', "Sastamala"),
    ('sauvo', "Sauvo"),
    ('savitaipale', "Savitaipale"),
    ('savonlinna', "Savonlinna"),
    ('savukoski', "Savukoski"),
    ('seinajoki', "Seinäjoki"),
    ('sievi', "Sievi"),
    ('siikainen', "Siikainen"),
    ('siikajoki', "Siikajoki"),
    ('siikalatva', "Siikalatva"),
    ('siilinjarvi', "Siilinjärvi"),
    ('simo', "Simo"),
    ('sipoo', "Sipoo"),
    ('siuntio', "Siuntio"),
    ('sodankyla', "Sodankylä"),
    ('soini', "Soini"),
    ('somero', "Somero"),
    ('sonkajarvi', "Sonkajärvi"),
    ('sotkamo', "Sotkamo"),
    ('sottunga', "Sottunga"),
    ('sulkava', "Sulkava"),
    ('sund', "Sund"),
    ('suomenniemi', "Suomenniemi"),
    ('suomussalmi', "Suomussalmi"),
    ('suonenjoki', "Suonenjoki"),
    ('sysma', "Sysmä"),
    ('sakyla', "Säkylä"),
    ('taipalsaari', "Taipalsaari"),
    ('taivalkoski', "Taivalkoski"),
    ('taivassalo', "Taivassalo"),
    ('tammela', "Tammela"),
    ('tampere', "Tampere"),
    ('tarvasjoki', "Tarvasjoki"),
    ('tervo', "Tervo"),
    ('tervola', "Tervola"),
    ('teuva', "Teuva"),
    ('tohmajarvi', "Tohmajärvi"),
    ('toholampi', "Toholampi"),
    ('toivakka', "Toivakka"),
    ('tornio', "Tornio"),
    ('turku', "Turku"),
    ('tuusniemi', "Tuusniemi"),
    ('tuusula', "Tuusula"),
    ('tyrnava', "Tyrnävä"),
    ('toysa', "Töysä"),
    ('ulvila', "Ulvila"),
    ('urjala', "Urjala"),
    ('utajarvi', "Utajärvi"),
    ('utsjoki', "Utsjoki"),
    ('uurainen', "Uurainen"),
    ('uusikaarlepyy', "Uusikaarlepyy"),
    ('uusikaupunki', "Uusikaupunki"),
    ('vaala', "Vaala"),
    ('vaasa', "Vaasa"),
    ('valkeakoski', "Valkeakoski"),
    ('valtimo', "Valtimo"),
    ('vantaa', "Vantaa"),
    ('varkaus', "Varkaus"),
    ('varpaisjarvi', "Varpaisjärvi"),
    ('vehmaa', "Vehmaa"),
    ('vesanto', "Vesanto"),
    ('vesilahti', "Vesilahti"),
    ('veteli', "Veteli"),
    ('vierema', "Vieremä"),
    ('vihanti', "Vihanti"),
    ('vihti', "Vihti"),
    ('viitasaari', "Viitasaari"),
    ('vimpeli', "Vimpeli"),
    ('virolahti', "Virolahti"),
    ('virrat', "Virrat"),
    ('vardo', "Vårdö"),
    ('vahakyro', "Vähäkyrö"),
    ('voyri-maksamaa', "Vöyri-Maksamaa"),
    ('yli-ii', "Yli-Ii"),
    ('ylitornio', "Ylitornio"),
    ('ylivieska', "Ylivieska"),
    ('ylojarvi', "Ylöjärvi"),
    ('ypaja', "Ypäjä"),
    ('ahtari', "Ähtäri"),
    ('aanekoski', "Äänekoski")
)
